﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using WarmlyShip;
using static System.Windows.Forms.DataFormats;

namespace WarmlyShip
{
	public partial class FormMapWithSetShip : Form
	{
		private readonly Dictionary<string, AbstractMap> _mapsDict = new()
		{
			{ "Простая карта", new SimpleMap() },
			{ "Вторая карта", new SecondMap() },
			{ "Последняя карта", new LastMap() }
		};

		private readonly MapsCollection _mapsCollection;

		public FormMapWithSetShip()
		{
			InitializeComponent();
			_mapsCollection = new MapsCollection(pictureBox.Width, pictureBox.Height);
			comboBoxSelectorMap.Items.Clear();
			foreach (var elem in _mapsDict)
			{
				comboBoxSelectorMap.Items.Add(elem.Key);
			}
		}

		private void ReloadMaps()
		{
			int index = listBoxMaps.SelectedIndex;
			listBoxMaps.Items.Clear();
			for (int i = 0; i < _mapsCollection.Keys.Count; i++)
			{
				listBoxMaps.Items.Add(_mapsCollection.Keys[i]);
			}
			if (listBoxMaps.Items.Count > 0 && (index == -1 || index >= listBoxMaps.Items.Count))
			{
				listBoxMaps.SelectedIndex = 0;
			}
			else if (listBoxMaps.Items.Count > 0 && index > -1 && index < listBoxMaps.Items.Count)
			{
				listBoxMaps.SelectedIndex = index;
			}
		}

		private void ButtonAddMap_Click(object sender, EventArgs e)
		{
			if (comboBoxSelectorMap.SelectedIndex == -1 || string.IsNullOrEmpty(textBoxNewMapName.Text))
			{
				MessageBox.Show("Не все данные заполнены", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
				return;
			}
			if (!_mapsDict.ContainsKey(comboBoxSelectorMap.Text))
			{
				MessageBox.Show("Нет такой карты", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
				return;
			}
			_mapsCollection.AddMap(textBoxNewMapName.Text, _mapsDict[comboBoxSelectorMap.Text]);
			ReloadMaps();
		}

		private void ListBoxMaps_SelectedIndexChanged(object sender, EventArgs e)
		{
			pictureBox.Image = _mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? string.Empty].ShowSet();
		}

		private void ButtonDeleteMap_Click(object sender, EventArgs e)
		{
			if (listBoxMaps.SelectedIndex == -1)
			{
				return;
			}
			if (MessageBox.Show($"Удалить карту {listBoxMaps.SelectedItem}?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
			{
				_mapsCollection.DelMap(listBoxMaps.SelectedItem?.ToString() ??
			   string.Empty);
				ReloadMaps();
			}
		}

		private void ButtonAddShip_Click(object sender, EventArgs e)
		{
			FormShipConfig formShipConfig = new();
			formShipConfig.AddEvent(AddShip);
			formShipConfig.Show();
		}

		private void AddShip(DrawingWarmlyShip ship)
		{
			if (listBoxMaps.SelectedIndex == -1)
			{
				return;
			}

			if (_mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? string.Empty] + new DrawningObjectShip(ship) > -1)
			{
				MessageBox.Show("Объект добавлен");
				pictureBox.Image = _mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? string.Empty].ShowSet();
			}
			else
			{
				MessageBox.Show("Не удалось добавить объект");
			}
		}

		private void ButtonRemoveShip_Click(object sender, EventArgs e)
		{
			if (listBoxMaps.SelectedIndex == -1)
			{
				return;
			}
			if (string.IsNullOrEmpty(maskedTextBoxPosition.Text))
			{
				return;
			}
			if (MessageBox.Show("Удалить объект?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
			{
				return;
			}
			int pos = Convert.ToInt32(maskedTextBoxPosition.Text);
			if (_mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? string.Empty] - pos != null)
			{
				MessageBox.Show("Объект удален");
				pictureBox.Image =
			   _mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? string.Empty].ShowSet();
			}
			else
			{
				MessageBox.Show("Не удалось удалить объект");
			}
		}

		private void ButtonShowStorage_Click(object sender, EventArgs e)
		{
			if (listBoxMaps.SelectedIndex == -1)
			{
				return;
			}
			pictureBox.Image = _mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? string.Empty].ShowSet();
		}

		private void ButtonShowOnMap_Click(object sender, EventArgs e)
		{
			if (listBoxMaps.SelectedIndex == -1)
			{
				return;
			}
			pictureBox.Image = _mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? string.Empty].ShowOnMap();
		}

		private void ButtonMove_Click(object sender, EventArgs e)
		{
			if (listBoxMaps.SelectedIndex == -1)
			{
				return;
			}
			//получаем имя кнопки
			string name = ((Button)sender)?.Name ?? string.Empty;
			Direction dir = Direction.None;
			switch (name)
			{
				case "buttonUp":
					dir = Direction.Up;
					break;
				case "buttonDown":
					dir = Direction.Down;
					break;
				case "buttonLeft":
					dir = Direction.Left;
					break;
				case "buttonRight":
					dir = Direction.Right;
					break;
			}
			pictureBox.Image = _mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? string.Empty].MoveObject(dir);
		}
	}
}
