﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace WarmlyShip
{
	public partial class FormShipConfig : Form
	{

		DrawingWarmlyShip _warmlyShip = null;

		private Action<DrawingWarmlyShip> EventAddShip;

		public FormShipConfig()
		{
			InitializeComponent();
			panelBlack.MouseDown += PanelColor_MouseDown;
			panelPink.MouseDown += PanelColor_MouseDown;
			panelGreen.MouseDown += PanelColor_MouseDown;
			panelCyan.MouseDown += PanelColor_MouseDown;
			panelRed.MouseDown += PanelColor_MouseDown;
			panelWhite.MouseDown += PanelColor_MouseDown;
			panelYellow.MouseDown += PanelColor_MouseDown;
			panelBlue.MouseDown += PanelColor_MouseDown;

			buttonCancel.Click += (s, e) => Close();
		}

		private void DrawShip()
		{
			Bitmap bmp = new(pictureBoxObject.Width, pictureBoxObject.Height);
			Graphics gr = Graphics.FromImage(bmp);
			_warmlyShip?.SetPosition(5, 5, pictureBoxObject.Width, pictureBoxObject.Height);
			_warmlyShip?.DrawTransport(gr);
			pictureBoxObject.Image = bmp;
		}

		public void AddEvent(Action<DrawingWarmlyShip> ev)
		{
			if (EventAddShip == null)
			{
				EventAddShip = ev;
			}
			else
			{
				EventAddShip += ev;
			}
		}

		private void LabelObject_MouseDown(object sender, MouseEventArgs e)
		{
			(sender as Label).DoDragDrop((sender as Label).Name, DragDropEffects.Move | DragDropEffects.Copy);
		}

		private void PanelObject_DragEnter(object sender, DragEventArgs e)
		{
			if (e.Data.GetDataPresent(DataFormats.Text))
			{
				e.Effect = DragDropEffects.Copy;
			}
			else
			{
				e.Effect = DragDropEffects.None;
			}
		}

		private void PanelObject_DragDrop(object sender, DragEventArgs e)
		{
			switch (e.Data.GetData(DataFormats.Text).ToString())
			{
				case "labelSimpleObject":
					_warmlyShip = new DrawingWarmlyShip((int)numericUpDownSpeed.Value, (int)numericUpDownWeight.Value, Color.White);
					break;
				case "labelModifiedObject":
					_warmlyShip = new DrawningMotorShip((int)numericUpDownSpeed.Value, (int)numericUpDownWeight.Value, Color.White, Color.Black, checkBoxTubes.Checked, checkBoxCistern.Checked);
					break;
			}
			DrawShip();
		}

		private void PanelColor_MouseDown(object sender, MouseEventArgs e)
		{
			(sender as Control).DoDragDrop((sender as Control).BackColor, DragDropEffects.Move | DragDropEffects.Copy);
		}

		private void LabelBaseColor_DragEnter(object sender, DragEventArgs e)
		{
			if (e.Data.GetDataPresent(typeof(Color)))
			{
				e.Effect = DragDropEffects.Copy;
			}
			else
			{
				e.Effect = DragDropEffects.None;
			}
		}

		private void LabelBaseColor_DragDrop(object sender, DragEventArgs e)
		{
			if (_warmlyShip == null) return;

			if (_warmlyShip is DrawningMotorShip)
			{
				_warmlyShip = ((DrawningMotorShip)_warmlyShip).ModifShip(bodyColor: (Color)e.Data.GetData(typeof(Color)));
			}
			else
			{
				_warmlyShip = _warmlyShip.ModifShip(bodyColor: (Color)e.Data.GetData(typeof(Color)));
			}

			DrawShip();
		}

		private void LabelDopColor_DragDrop(object sender, DragEventArgs e)
		{
			if (_warmlyShip == null || !(_warmlyShip is DrawningMotorShip)) return;

			_warmlyShip = ((DrawningMotorShip)_warmlyShip).ModifShip(dopColor: (Color)e.Data.GetData(typeof(Color)));

			DrawShip();
		}

		private void ButtonOk_Click(object sender, EventArgs e)
		{
			EventAddShip?.Invoke(_warmlyShip);
			Close();
		}

	}
}
